% GAdemo: Demonstration of GAMIN minimization routine
%
% Gives user choice of three standard problems:
%  (1) L2norm (root sum of squares)
%  (2) Rosenbrok function  
%  (3) Corana function
%
% The first of these is quite easy, but the other two are standard
% test problems because they are hard.  Do not be disappointed by
% failure to get as good a solution as one typically does on
% nicely behaved, globally convex functions.
%
% Michael Gordy <mgordy@frb.gov> 26 January 2001

% Get default options structure.
optu=gadefaultoptions;

demochoice=menu('Choose a GA demo problem:','L2norm','Rosenbrok','Corana');

switch demochoice
  case 1
    % Minimize root sum of squares over [-5,5] in each dimension.
    funstr='L2norm';  K=4;
    optu.Epsilon=1e-6;
    optu.DisplayIterations=200;
    optu.VectorizedFunction='on';
    optu.ReproductionSelection='roulette';
    optu.MutationRate=0.03;
    parspace=repmat([-5 5],K,1);
    beta0=zeros(K,1);
    %
  case 2
    % Rosenbrok test program in K dimensions.
    %   Global min at ones(K,1).  
    funstr='rosenbrok';  K=4;
    optu.Epsilon=1e-6;
    optu.VectorizedFunction='on';
    optu.ReproductionSelection='tournament';
    optu.GenerationSize=40;
    optu.CrossoverRate=0.9;
    optu.MutationRate=-1;            % Adaptive mutation rate
    optu.MaximumIterations=120000;
    optu.StopIterations=10000;
    optu.DisplayIterations=500;
    parspace=repmat([-10 10],K,1);
    beta0=ones(K,1);
    %
  case 3
    % Corana problem in K dimensions
    %   Global min in [-0.05,0.05]^K
    %   Search over [-1000,1000] in each dimension.
    funstr='corana';  K=3;
    optu.Epsilon=1e-6;
    optu.VectorizedFunction='on';
    optu.MaximumIterations=100000;
    optu.GenerationSize=20;
    optu.CrossoverRate=-1;            % Bookers VCO
    optu.MutationRate=-1;             % Adaptive mutation rate
    optu.ReproductionSelection='tournament';
    optu.StopIterations=10000;
    optu.DisplayIterations=1000;
    parspace=repmat([-1000 1000],K,1);
    beta0=zeros(K,1);
end

% Run the GA
[betaga,sc]=gamin(funstr,parspace,[],optu);

% Print out results.
brkstr=repmat('-',1,49);
titlestr=['\nGAMIN Solution of %2.0f-dimensional ',...
                     upper(funstr),' function'];
disp(sprintf(titlestr,K)),  disp(brkstr)
disp(sprintf('\t\t  Optimum  GA-Best'))
disp(sprintf('\t\t %8.5f %8.5f\n',[beta0 betaga]'))
disp(brkstr)
disp([upper(funstr),sprintf(':\t %8.5f %8.5f',...
                            [feval(funstr,beta0) feval(funstr,betaga)])])
disp([brkstr,sprintf('\n\n')])
